@@ -7,10 +7,10 @@
  * and reworked somewhat to fit with my coding style.
  *
  * The canonical version of this file is maintained in the rra-c-util package,
- * which can be found at <http://www.eyrie.org/~eagle/software/rra-c-util/>.
+ * which can be found at <https://www.eyrie.org/~eagle/software/rra-c-util/>.
  *
- * Copyright 1997, 2000, 2001, 2002 Benjamin Sittler
- * Copyright 2008 Russ Allbery <eagle@eyrie.org>
+ * Copyright 2008, 2017 Russ Allbery <eagle@eyrie.org>
+ * Copyright 1997, 2000-2002 Benjamin Sittler
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -29,6 +29,8 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
+ *
+ * SPDX-License-Identifier: MIT
  */
 
 #include <config.h>
@@ -51,6 +53,10 @@ int test_getopt(int, char **, const char *);
 # define opterr test_opterr
 # define optopt test_optopt
 # define optarg test_optarg
+extern int test_optind;
+extern int test_opterr;
+extern int test_optopt;
+extern char *test_optarg;
 #endif
 
 /* Initialize global interface variables. */
