@@ -6,12 +6,12 @@
  *
  * Written by Russ Allbery <eagle@eyrie.org>
  * Based on work by Anton Ushakov
+ * Copyright 2015-2016 Russ Allbery <eagle@eyrie.org>
  * Copyright 2016 Dropbox, Inc.
- * Copyright 2015, 2016 Russ Allbery <eagle@eyrie.org>
- * Copyright 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2012, 2013,
- *     2014 The Board of Trustees of the Leland Stanford Junior University
+ * Copyright 2002-2010, 2012-2014
+ *     The Board of Trustees of the Leland Stanford Junior University
  *
- * See LICENSE for licensing terms.
+ * SPDX-License-Identifier: MIT
  */
 
 #include <config.h>
@@ -217,7 +217,7 @@ create_argv_command(struct rule *rule, struct process *process,
         req_argv[1] = xstrdup("-u");
         req_argv[2] = xstrdup(rule->sudo_user);
         req_argv[3] = xstrdup("--");
-        req_argv[4] = rule->program;
+        req_argv[4] = xstrdup(rule->program);
         j = 5;
     } else {
         program = strrchr(rule->program, '/');
@@ -337,7 +337,7 @@ server_run_command(struct client *client, struct config *config,
     }
 
     /* Neither the command nor the subcommand may ever contain nuls. */
-    for (i = 0; argv[i] != NULL && i < 2; i++) {
+    for (i = 0; i < 2 && argv[i] != NULL; i++) {
         if (memchr(argv[i]->iov_base, '\0', argv[i]->iov_len)) {
             notice("%s from user %s contains nul octet",
                    (i == 0) ? "command" : "subcommand", user);
