@@ -2,11 +2,11 @@
  * Test suite for network client and read/write functions.
  *
  * The canonical version of this file is maintained in the rra-c-util package,
- * which can be found at <http://www.eyrie.org/~eagle/software/rra-c-util/>.
+ * which can be found at <https://www.eyrie.org/~eagle/software/rra-c-util/>.
  *
  * Written by Russ Allbery <eagle@eyrie.org>
- * Copyright 2005, 2013, 2014, 2016 Russ Allbery <eagle@eyrie.org>
- * Copyright 2009, 2010, 2011, 2012, 2013
+ * Copyright 2005, 2013-2014, 2016-2018 Russ Allbery <eagle@eyrie.org>
+ * Copyright 2009-2013
  *     The Board of Trustees of the Leland Stanford Junior University
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -26,6 +26,8 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
+ *
+ * SPDX-License-Identifier: MIT
  */
 
 #include <config.h>
@@ -47,7 +49,7 @@
  * and expects to always succeed on the connection, taking the source address
  * to pass into network_client_create.
  */
-static void
+static void __attribute__((__noreturn__))
 client_create_writer(const char *source)
 {
     socket_type fd;
@@ -77,7 +79,7 @@ client_create_writer(const char *source)
  * sleeps for 10 seconds before sending another string so that timeouts can be
  * tested.  Meant to be run in a child process.
  */
-static void
+static void __attribute__((__noreturn__))
 client_delay_writer(const char *host)
 {
     socket_type fd;
@@ -100,7 +102,7 @@ client_delay_writer(const char *host)
  * Used to test network_write.  Connects, reads 64KB from the network, then
  * sleeps before reading another 64KB.  Meant to be run in a child process.
  */
-static void
+static void __attribute__((__noreturn__))
 client_delay_reader(const char *host)
 {
     char *buffer;
@@ -376,10 +378,6 @@ test_network_write(void)
      */
     const size_t bufsize = 15 * 1024 * 1024;
 
-    /* Create the data that we're going to send. */
-    buffer = bmalloc(bufsize);
-    memset(buffer, 'a', bufsize);
-
     /* Create the listening socket. */
     fd = network_bind_ipv4(SOCK_STREAM, "127.0.0.1", 11119);
     if (fd == INVALID_SOCKET)
@@ -396,6 +394,10 @@ test_network_write(void)
         client_delay_reader("127.0.0.1");
     }
 
+    /* Create the data that we're going to send. */
+    buffer = bmalloc(bufsize);
+    memset(buffer, 'a', bufsize);
+
     /* Set an alarm just in case our timeouts don't work. */
     alarm(10);
 
